@@ -6,12 +6,15 @@ require 'date'
 class CycleTimeConfig
   include SelfOrIssueDispatcher
 
-  attr_reader :label, :parent_config
+  attr_reader :label, :parent_config, :settings, :file_system
+
+  def initialize parent_config:, label:, block:, settings:, file_system: nil, today: Date.today
 
-  def initialize parent_config:, label:, block:, file_system: nil, today: Date.today
     @parent_config = parent_config
     @label = label
     @today = today
+    @settings = settings
+    @cache_cycletime_calculations = settings['cache_cycletime_calculations']
 
     # If we hit something deprecated and this is nil then we'll blow up. Although it's ugly, this
     # may make it easier to find problems in the test code ;-)
@@ -63,6 +66,10 @@ class CycleTimeConfig
   end
 
   def started_stopped_changes issue
+    cache_key = "#{issue.key}:#{issue.board.id}"
+    last_result = (@cache ||= {})[cache_key]
+    return *last_result if last_result && @cache_cycletime_calculations
+
     started = @start_at.call(issue)
     stopped = @stop_at.call(issue)
 
@@ -80,7 +87,15 @@ class CycleTimeConfig
     # for the start and not have it conflict.
     started = nil if started&.time == stopped&.time
 
-    [started, stopped]
+    result = [started, stopped]
+    if last_result && result != last_result
+      @file_system.error(
+        "Calculation mismatch; this could break caching. #{issue.inspect} new=#{result.inspect}, " \
+          "previous=#{last_result.inspect}"
+      )
+    end
+    @cache[cache_key] = result
+    result
   end
 
   def started_stopped_times issue
@@ -88,6 +103,10 @@ class CycleTimeConfig
     [started&.time, stopped&.time]
   end
 
+  def flush_cache
+    @cache = nil
+  end
+
   def started_stopped_dates issue
     started_time, stopped_time = started_stopped_times(issue)
     [started_time&.to_date, stopped_time&.to_date]
