@@ -19,9 +19,10 @@ class Issue
 
     # There are cases where we create an Issue of fragments like linked issues and those won't have
     # changelogs.
-    return unless @raw['changelog']
+    load_history_into_changes if @raw['changelog']
 
-    load_history_into_changes
+    # As above with fragments, there may not be a fields section
+    return unless @raw['fields']
 
     # If this is an older pull of data then comments may not be there.
     load_comments_into_changes if @raw['fields']['comment']
@@ -44,9 +45,11 @@ class Issue
   def key = @raw['key']
 
   def type = @raw['fields']['issuetype']['name']
-
   def type_icon_url = @raw['fields']['issuetype']['iconUrl']
 
+  def priority_name = @raw['fields']['priority']['name']
+  def priority_url = @raw['fields']['priority']['iconUrl']
+
   def summary = @raw['fields']['summary']
 
   def labels = @raw['fields']['labels'] || []
@@ -150,7 +153,7 @@ class Issue
   # Are we currently in this status? If yes, then return the most recent status change.
   def currently_in_status *status_names
     change = most_recent_status_change
-    return false if change.nil?
+    return nil if change.nil?
 
     change if change.current_status_matches(*status_names)
   end
@@ -160,7 +163,7 @@ class Issue
     category_ids = find_status_category_ids_by_names category_names
 
     change = most_recent_status_change
-    return false if change.nil?
+    return nil if change.nil?
 
     status = find_or_create_status id: change.value_id, name: change.value
     change if status && category_ids.include?(status.category.id)
@@ -205,8 +208,16 @@ class Issue
     nil
   end
 
+  def first_time_visible_on_board
+    first_time_in_status(*board.visible_columns.collect(&:status_ids).flatten)
+  end
+
   def parse_time text
-    Time.parse(text).getlocal(@timezone_offset)
+    if text.is_a? String
+      Time.parse(text).getlocal(@timezone_offset)
+    else
+      Time.at(text / 1000).getlocal(@timezone_offset)
+    end
   end
 
   def created
@@ -227,7 +238,11 @@ class Issue
   end
 
   def assigned_to
-    @raw['fields']&.[]('assignee')&.[]('displayName')
+    @raw['fields']['assignee']&.[]('displayName')
+  end
+
+  def assigned_to_icon_url
+    @raw['fields']['assignee']&.[]('avatarUrls')&.[]('16x16')
   end
 
   # Many test failures are simply unreadable because the default inspect on this class goes
@@ -315,7 +330,7 @@ class Issue
 
     # This mock change is to force the writing of one last entry at the end of the time range.
     # By doing this, we're able to eliminate a lot of duplicated code in charts.
-    mock_change = ChangeItem.new time: end_time, author: '', artificial: true, raw: { 'field' => '' }
+    mock_change = ChangeItem.new time: end_time, artificial: true, raw: { 'field' => '' }, author_raw: nil
 
     (changes + [mock_change]).each do |change|
       previous_was_active = false if check_for_stalled(
@@ -335,7 +350,7 @@ class Issue
         end
       elsif change.link?
         # Example: "This issue is satisfied by ANON-30465"
-        unless /^This issue (?<link_text>.+) (?<issue_key>.+)$/ =~ (change.value || change.old_value)
+        unless /^This (?<_>issue|work item) (?<link_text>.+) (?<issue_key>.+)$/ =~ (change.value || change.old_value)
           puts "Issue(#{key}) Can't parse link text: #{change.value || change.old_value}"
           next
         end
@@ -462,8 +477,6 @@ class Issue
   end
 
   def expedited?
-    return false unless @board&.project_config
-
     names = @board.project_config.settings['expedited_priority_names']
     return false unless names
 
@@ -580,7 +593,7 @@ class Issue
     /(?<project_code1>[^-]+)-(?<id1>.+)/ =~ key
     /(?<project_code2>[^-]+)-(?<id2>.+)/ =~ other.key
     comparison = project_code1 <=> project_code2
-    comparison = id1 <=> id2 if comparison.zero?
+    comparison = id1.to_i <=> id2.to_i if comparison.zero?
     comparison
   end
 
@@ -600,7 +613,7 @@ class Issue
 
   def dump
     result = +''
-    result << "#{key} (#{type}): #{compact_text summary, 200}\n"
+    result << "#{key} (#{type}): #{compact_text summary, max: 200}\n"
 
     assignee = raw['fields']['assignee']
     result << "  [assignee] #{assignee['name'].inspect} <#{assignee['emailAddress']}>\n" unless assignee.nil?
@@ -611,9 +624,13 @@ class Issue
     end
     history = [] # time, type, detail
 
-    started_at, stopped_at = board.cycletime.started_stopped_times(self)
-    history << [started_at, nil, '↓↓↓↓ Started here ↓↓↓↓', true] if started_at
-    history << [stopped_at, nil, '↑↑↑↑ Finished here ↑↑↑↑', true] if stopped_at
+    if board.cycletime
+      started_at, stopped_at = board.cycletime.started_stopped_times(self)
+      history << [started_at, nil, '↓↓↓↓ Started here ↓↓↓↓', true] if started_at
+      history << [stopped_at, nil, '↑↑↑↑ Finished here ↑↑↑↑', true] if stopped_at
+    else
+      result << "  Unable to determine start/end times as board #{board.id} has no cycletime specified\n"
+    end
 
     @discarded_change_times&.each do |time|
       history << [time, nil, '↑↑↑↑ Changes discarded ↑↑↑↑', true]
@@ -669,9 +686,8 @@ class Issue
   def done?
     if artificial? || board.cycletime.nil?
       # This was probably loaded as a linked issue, which means we don't know what board it really
-      # belonged to. The best we can do is look at the status category. This case should be rare but
-      # it can happen.
-      status.category.name == 'Done'
+      # belonged to. The best we can do is look at the status key
+      status.category.done?
     else
       board.cycletime.done? self
     end
@@ -681,45 +697,56 @@ class Issue
     @changes.select { |change| change.status? }
   end
 
-  private
+  def sprints
+    sprint_ids = []
 
-  def assemble_author raw
-    raw['author']&.[]('displayName') || raw['author']&.[]('name') || 'Unknown author'
+    changes.each do |change|
+      next unless change.sprint?
+
+      sprint_ids << change.raw['to'].split(/\s*,\s*/).collect { |id| id.to_i }
+    end
+    sprint_ids.flatten!
+
+    board.sprints.select { |s| sprint_ids.include? s.id }
   end
 
+  def compact_text text, max: 60
+    return '' if text.nil?
+
+    if text.is_a? Hash
+      # We can't effectively compact it but we can convert it into a string.
+      text = @board.project_config.atlassian_document_format.to_html(text)
+    else
+      text = text.gsub(/\s+/, ' ').strip
+      text = "#{text[0...max]}..." if text.length > max
+    end
+    text
+  end
+
+  private
+
   def load_history_into_changes
     @raw['changelog']['histories']&.each do |history|
       created = parse_time(history['created'])
 
-      # It should be impossible to not have an author but we've seen it in production
-      author = assemble_author history
       history['items']&.each do |item|
-        @changes << ChangeItem.new(raw: item, time: created, author: author)
+        @changes << ChangeItem.new(raw: item, time: created, author_raw: history['author'])
       end
     end
   end
 
   def load_comments_into_changes
     @raw['fields']['comment']['comments']&.each do |comment|
-      raw = {
+      raw = comment.merge({
         'field' => 'comment',
         'to' => comment['id'],
         'toString' =>  comment['body']
-      }
-      author = assemble_author comment
+      })
       created = parse_time(comment['created'])
-      @changes << ChangeItem.new(raw: raw, time: created, author: author, artificial: true)
+      @changes << ChangeItem.new(raw: raw, time: created, artificial: true, author_raw: comment['author'])
     end
   end
 
-  def compact_text text, max = 60
-    return nil if text.nil?
-
-    text = text.gsub(/\s+/, ' ').strip
-    text = "#{text[0..max]}..." if text.length > max
-    text
-  end
-
   def sort_changes!
     @changes.sort! do |a, b|
       # It's common that a resolved will happen at the same time as a status change.
@@ -737,6 +764,9 @@ class Issue
     first_status = nil
     first_status_id = nil
 
+    # There won't be a created timestamp in cases where this was a linked issue
+    return unless @raw['fields']['created']
+
     created_time = parse_time @raw['fields']['created']
     first_change = @changes.find { |change| change.field == field_name }
     if first_change.nil?
@@ -750,7 +780,9 @@ class Issue
       first_status = first_change.old_value
       first_status_id = first_change.old_value_id
     end
-    ChangeItem.new time: created_time, artificial: true, author: author, raw: {
+
+    creator = raw['fields']['creator']
+    ChangeItem.new time: created_time, artificial: true, author_raw: creator, raw: {
       'field' => field_name,
       'to' => first_status_id,
       'toString' => first_status
